import numpy as np
import pandas as pd
import sys
sys.path.insert(0, dirName) # dirName = location where the files listed here are located

class MiscUnc():
    '''Uncertainty calculations for power measurements.'''
    # List of power measurements that do not use a CT (current transducer)
    noCT = ['oau_fan_power','ahu1_fan_power','ahu2_fan_power','exf1_fan_power','exf2_fan_power',
            'pump1_power','pump2_power','pump3_power','pump4_power','pump5_power']
    # List of power measurements that are taken at a faster sample rate than other power measurements and then 
    # integrated to return the power use over a 9 second interval
    fast = ['ahu1_heat_power','ahu2_heat_power','vav1_heat_power','vav2_heat_power','vav3_heat_power',
            'vav4_heat_power','zs1_heat_power','zs2_heat_power','zs3_heat_power','zs4_heat_power']
    def uPowerV(self,df,mData,pn):
        '''Calculate the measurement uncertainty of the power measurement in V. pn is the specific sensor, such as
           oau_fan_power. mData is the metadata file.'''
        # Get the measurements
        P = df[pn] # W
        # Get the scaling coefficients for that measurement
        a0 = float(mData.loc[pn].scaling_coeff_a0) # W
        a1 = float(mData.loc[pn].scaling_coeff_a1) # W/V
        V = (P-a0)/a1 # Calculate the raw voltage from the value P in W using the scaling coefficients
        if pn in MiscUnc.fast:
            # The measurement is in the fast category
            V = V/9 # the V calculated is the integrated V; I want to calculate the uncertainty per each
                    # individual voltage measurement, not the integrated measurement. That's accounted for in the
                    # overall uncertainty 
        if pn in MiscUnc.noCT:
            # The measurement is in the noCT category, so the manufacturer uncertainty is different
            uManuf = 0.002*V
        else:
            uManuf = 0.0047*V
        uCard = 0.00246 # V, the measurement uncertainty of the DAQ card, PXIe-4300
        uV = np.sqrt(uCard**2 + uManuf**2) # Total uncertainty
        return uV,a1/1000,V
    
    def uPower(self,df,mData,pn,N = 9000):
        '''This is the main function to call. Calculate the uncertainty of the measurement pn in W and V. df is the scaled
           data, mData is the metadata, pn is the power sensor measurement (e.g., oau_fan_power), and N is the number of samples
           used in the integration for the fast measurements.'''
        uV,a1,V = self.uPowerV(df,mData,pn) # V, kW/V
        if pn in MiscUnc.fast:
            # The sensor is in the fast category; the uncertainty is the accumulation over 9 seconds
            uVs = 0.001*uV*np.sqrt(N-0.5) 
            u = a1*uVs
        else:
            u = a1*uV
        df['u_'+pn]=u # kW
        df['V_'+pn] = V
        return df 
